﻿using TypographyContracts.BindingModels;
using TypographyContracts.BusinessLogicsContracts;
using Microsoft.Extensions.Logging;

namespace TypographyView
{
    public partial class FormPrinteds : Form
    {
        private readonly ILogger _logger;
        private readonly IPrintedLogic _logic;

        public FormPrinteds(ILogger<FormPrinteds> logger, IPrintedLogic logic)
        {
            InitializeComponent();
            _logger = logger;
            _logic = logic;
        }
        private void LoadData()
        {
            try
            {
                var list = _logic.ReadList(null);
                if (list != null)
                {
                    dataGridViewPrinteds.DataSource = list;
                    dataGridViewPrinteds.Columns["Id"].Visible = false;
                    dataGridViewPrinteds.Columns["PrintedComponents"].Visible = false;
                    dataGridViewPrinteds.Columns["PrintedName"].AutoSizeMode = DataGridViewAutoSizeColumnMode.Fill;
                }
                _logger.LogInformation("Загрузка печатной продукции");
            }
            catch (Exception ex)
            {
                _logger.LogError(ex, "Ошибка загрузки печатной продукции");
                MessageBox.Show(ex.Message, "Ошибка", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
        }
        private void buttonAdd_Click(object sender, EventArgs e)
        {
            var service = Program.ServiceProvider?.GetService(typeof(FormPrinted));
            if (service is FormPrinted form)
            {
                if (form.ShowDialog() == DialogResult.OK)
                {
                    LoadData();
                }
            }
        }
        private void buttonUpd_Click(object sender, EventArgs e)
        {
            if (dataGridViewPrinteds.SelectedRows.Count == 1)
            {
                var service = Program.ServiceProvider?.GetService(typeof(FormPrinted));
                if (service is FormPrinted form)
                {
                    form.Id = Convert.ToInt32(dataGridViewPrinteds.SelectedRows[0].Cells["Id"].Value);
                    if (form.ShowDialog() == DialogResult.OK)
                    {
                        LoadData();
                    }
                }
            }
        }
        private void buttonDel_Click(object sender, EventArgs e)
        {
            if (dataGridViewPrinteds.SelectedRows.Count == 1)
            {
                if (MessageBox.Show("Удалить запись?", "Вопрос", MessageBoxButtons.YesNo, MessageBoxIcon.Question) ==
                    DialogResult.Yes)
                {
                    int id = Convert.ToInt32(dataGridViewPrinteds.SelectedRows[0].Cells["Id"].Value);
                    _logger.LogInformation("Удаление компонента");
                    try
                    {
                        if (!_logic.Delete(new PrintedBindingModel { Id = id }))
                        {
                            throw new Exception("Ошибка при удалении. Дополнительная информация в логах.");
                        }
                        LoadData();
                    }
                    catch (Exception ex)
                    {
                        _logger.LogError(ex, "Ошибка удаления печатной продукции");
                        MessageBox.Show(ex.Message, "Ошибка", MessageBoxButtons.OK, MessageBoxIcon.Error);
                    }
                }
            }
        }
        private void buttonRef_Click(object sender, EventArgs e)
        {
            LoadData();
        }
        private void FormPrinteds_Load(object sender, EventArgs e)
        {
            LoadData();
        }
    }
}
