﻿using Microsoft.EntityFrameworkCore;
using TypographyContracts.BindingModels;
using TypographyContracts.SearchModels;
using TypographyContracts.StoragesContracts;
using TypographyContracts.ViewModels;
using TypographyDatabaseImplement.Models;

namespace TypographyDatabaseImplement.Implements
{
    public class OrderStorage : IOrderStorage
    {
        public OrderViewModel? GetElement(OrderSearchModel model)
        {
            if (!model.Id.HasValue)
            {
                return null;
            }
            using var context = new TypographyDatabase();
            return context.Orders
                .Include(x => x.Printed)
                .FirstOrDefault(x => (model.Id.HasValue && x.Id == model.Id))?.GetViewModel;
        }
        public List<OrderViewModel> GetFilteredList(OrderSearchModel model)
        {
            if (!model.DateFrom.HasValue || !model.DateTo.HasValue)
            {
                return new();
            }
            using var context = new TypographyDatabase();
            return context.Orders
                .Include(x => x.Printed)
                .Where(x => x.DateCreate >= model.DateFrom && x.DateCreate <= model.DateTo)
                .Select(x => x.GetViewModel)
                .ToList();
        }
        public List<OrderViewModel> GetFullList()
        {
            using var context = new TypographyDatabase();
            return context.Orders
                .Include(x => x.Printed)
                .Select(x => x.GetViewModel)
                .ToList();
        }
        public OrderViewModel? Insert(OrderBindingModel model)
        {
            using var context = new TypographyDatabase();
            var newOrder = Order.Create(model, context);
            if (newOrder == null)
            {
                return null;
            }
            context.Orders.Add(newOrder);
            context.SaveChanges();
            return newOrder.GetViewModel;
        }
        public OrderViewModel? Update(OrderBindingModel model)
        {
            using var context = new TypographyDatabase();
            var order = context.Orders.FirstOrDefault(x => x.Id == model.Id);
            if (order == null)
            {
                return null;
            }
            order.Update(model);
            context.SaveChanges();
            return order.GetViewModel;
        }
        public OrderViewModel? Delete(OrderBindingModel model)
        {
            using var context = new TypographyDatabase();
            var order = context.Orders.FirstOrDefault(rec => rec.Id == model.Id);
            if (order != null)
            {
                context.Orders.Remove(order);
                context.SaveChanges();
                return order.GetViewModel;
            }
            return null;
        }
    }
}