﻿using TypographyContracts.BindingModels;
using TypographyContracts.SearchModels;
using TypographyContracts.StoragesContracts;
using TypographyContracts.ViewModels;
using TypographyFileImplement.Models;

namespace TypographyFileImplement.Implements
{
    public class OrderStorage : IOrderStorage
    {
        private readonly DataFileSingleton source;
        public OrderStorage()
        {
            source = DataFileSingleton.GetInstance();
        }
        public List<OrderViewModel> GetFullList()
        {
            return source.Orders
                .Select(x => DefinePrintedName(x.GetViewModel))
                .ToList();
        }
        private OrderViewModel DefinePrintedName(OrderViewModel model)
        {
            string? printedName = source.Printeds.FirstOrDefault(x => model.PrintedId == x.Id)?.PrintedName;
            if (printedName != null)
            {
                model.PrintedName = printedName;
            }
            return model;
        }
        public List<OrderViewModel> GetFilteredList(OrderSearchModel model)
        {
            if (!model.DateFrom.HasValue || !model.DateTo.HasValue)
            {
                return new();
            }
            return source.Orders
                .Where(x => x.DateCreate >= model.DateFrom && x.DateCreate <= model.DateTo)
                .Select(x => DefinePrintedName(x.GetViewModel))
                .ToList();
        }
        public OrderViewModel? GetElement(OrderSearchModel model)
        {
            if (!model.Id.HasValue)
            {
                return null;
            }
            return DefinePrintedName(source.Orders
                .FirstOrDefault(x => (model.Id.HasValue && x.Id == model.Id))?.GetViewModel);
        }
        public OrderViewModel? Insert(OrderBindingModel model)
        {
            model.Id = source.Orders.Count > 0 ? source.Orders.Max(x => x.Id) + 1 : 1;
            var newOrder = Order.Create(model);
            if (newOrder == null)
            {
                return null;
            }
            source.Orders.Add(newOrder);
            source.SaveOrders();
            return DefinePrintedName(newOrder.GetViewModel);
        }
        public OrderViewModel? Update(OrderBindingModel model)
        {
            var order = source.Orders.FirstOrDefault(x => x.Id == model.Id);
            if (order == null)
            {
                return null;
            }
            order.Update(model);
            source.SaveOrders();
            return DefinePrintedName(order.GetViewModel);
        }
        public OrderViewModel? Delete(OrderBindingModel model)
        {
            var element = source.Orders.FirstOrDefault(x => x.Id == model.Id);
            if (element != null)
            {
                source.Orders.Remove(element);
                source.SaveOrders();
                return DefinePrintedName(element.GetViewModel);
            }
            return null;
        }
    }
}