﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Xml.Linq;
using TypographyContracts.BindingModels;
using TypographyContracts.ViewModels;
using TypographyDataModels.Models;

namespace TypographyFileImplement.Models
{
    public class Printed : IPrintedModel
    {
        public int Id { get; private set; }
        public string PrintedName { get; private set; } = string.Empty;
        public double Price { get; private set; }
        public Dictionary<int, int> Components { get; private set; } = new();
        private Dictionary<int, (IComponentModel, int)>? _productComponents = null;
        public Dictionary<int, (IComponentModel, int)> PrintedComponents
        {
            get
            {
                if (_productComponents == null)
                {
                    var source = DataFileSingleton.GetInstance();
                    _productComponents = Components.ToDictionary(x => x.Key, y =>
                        ((source.Components.FirstOrDefault(z => z.Id == y.Key) as IComponentModel)!, y.Value));
                }
                return _productComponents;
            }
        }
        public static Printed? Create(PrintedBindingModel model)
        {
            if (model == null)
            {
                return null;
            }
            return new Printed()
            {
                Id = model.Id,
                PrintedName = model.PrintedName,
                Price = model.Price,
                Components = model.PrintedComponents.ToDictionary(x => x.Key, x => x.Value.Item2)
            };
        }
        public static Printed? Create(XElement element)
        {
            if (element == null)
            {
                return null;
            }
            return new Printed()
            {
                Id = Convert.ToInt32(element.Attribute("Id")!.Value),
                PrintedName = element.Element("ProductName")!.Value,
                Price = Convert.ToDouble(element.Element("Price")!.Value),
                Components = element.Element("ProductComponents")!.Elements("ProductComponent").ToDictionary(x =>
                    Convert.ToInt32(x.Element("Key")?.Value), x =>
                        Convert.ToInt32(x.Element("Value")?.Value))
            };
        }
        public void Update(PrintedBindingModel model)
        {
            if (model == null)
            {
                return;
            }
            PrintedName = model.PrintedName;
            Price = model.Price;
            Components = model.PrintedComponents.ToDictionary(x => x.Key, x => x.Value.Item2);
            _productComponents = null;
        }
        public PrintedViewModel GetViewModel => new()
        {
            Id = Id,
            PrintedName = PrintedName,
            Price = Price,
            PrintedComponents = PrintedComponents
        };
        public XElement GetXElement => new("Product",
            new XAttribute("Id", Id),
            new XElement("ProductName", PrintedName),
            new XElement("Price", Price.ToString()),
            new XElement("ProductComponents", Components.Select(x =>
                new XElement("ProductComponent",
                new XElement("Key", x.Key),
                new XElement("Value", x.Value)))
                .ToArray()));
    }
}